/*
 Mit Hilfe des Programms soll eine Pruefung durchgefuehrt werden.
 Die Pruefung besteht aus einem Quiz, das eine Menge an
 Prueflingen zu absolvieren haben. Ein Quiz besteht aus einer
 Menge an Fragen. Bei den Fragen handelt es sich ausschliesslich
 um Wahr/Falsch-Aussagen, d.h. ein Pruefling muss jeweils
 angeben, ob die Aussage wahr oder falsch ist. Bei einer
 richtigen Antwort bekommt er eine der Frage zugeordneten Punktzahl.

 Eine Pruefung laeuft so ab, dass zunaechst das Quiz vorbereitet wird,
 d.h. es werden eine Menge an Fragen eingeben.
 Anschliessend wird die Pruefung durchgefuehrt, d.h. die Prueflinge
 muessen der Reihe nach die Fragen beantworten. 
 Danach wird eine Rangliste nach den erreichten Punkten erzeugt
 und die Ergebnisse der Pruefung ausgegeben.
 */
class Frage {
	String frage; // Fragetext

	boolean richtig; // richtig oder falsch

	int punkte; // zu erreichende Punktzahl

	Frage(String frage, boolean richtig, int punkte) {
		this.frage = frage;
		this.richtig = richtig;
		this.punkte = punkte;
	}

	String getFrage() {
		return this.frage;
	}

	boolean isRichtig() {
		return this.richtig;
	}

	int getPunkte() {
		return this.punkte;
	}
}

class Quiz {
	Frage[] fragen; // Menge an Fragen

	String titel; // Titel des Quizes

	int aktuellerIndex; // aktuelle Anzahl an Fragen-1

	int naechsterIndex; // Schleifenvariable

	// Konstruktor
	Quiz(String titel, int maxFragen) {
		this.titel = titel;
		this.aktuellerIndex = -1;
		this.fragen = new Frage[maxFragen];
		for (int i = 0; i < this.fragen.length; i++)
			this.fragen[i] = null;
		this.naechsterIndex = -1;
	}

	String getTitel() {
		return this.titel;
	}

	// Frage hinzufuegen
	void neueFrage(Frage f) {
		if (this.aktuellerIndex < this.fragen.length - 1) {
			this.fragen[++this.aktuellerIndex] = f;
		}
	}

	// liefert zyklisch die naechste Frage oder null, falls keine (mehr)
	// vorhanden ist
	Frage liefereNaechsteFrage() {
		if (this.fragen.length == 0)
			return null;
		Frage f = null;
		if (this.naechsterIndex < this.aktuellerIndex) {
			f = this.fragen[++this.naechsterIndex];
		} else {
			this.naechsterIndex = -1; // bereite naechsten Zyklus vor
		}
		return f;
	}
}

class Pruefling {
	String name; // Name der Person

	int punkte; // bisher erzielte Punkte

	Pruefling(String name) {
		this.name = name;
		this.punkte = 0;
	}

	String getName() {
		return this.name;
	}

	int getPunkte() {
		return this.punkte;
	}

	void neuePunkte(int anzahl) {
		this.punkte += anzahl;
	}
}

public class Pruefung {

	// Hauptprogramm
	public static void main(String[] args) {
		Pruefung klausur = new Pruefung();
		klausur.vorbereiten();
		klausur.durchfuehren();
		klausur.ergebnisseBekanntgeben();
	}

	Quiz pruefung;

	Pruefling[] studenten;

	Pruefung() {
		this.pruefung = null;
		this.studenten = null;
	}

	Pruefung(Quiz quiz, Pruefling[] prueflinge) {
		this.pruefung = quiz;
		this.studenten = prueflinge;
	}

	void vorbereiten() {
		IO.println("Fragen eingeben");
		IO.println("----------------");
		String titel = IO.readString("Titel des Quizes: ");
		int anzahl = IO.readInt("Anzahl Fragen: ");
		this.pruefung = new Quiz(titel, anzahl);
		for (int i = 0; i < anzahl; i++) {
			String text = IO.readString("Frage " + (i + 1) + ": ");
			boolean wahr = IO.readChar("wahr/falsch (w/f)?") == 'w';
			int punkte = IO.readInt("Erreichbare Punkte: ");
			Frage f = new Frage(text, wahr, punkte);
			this.pruefung.neueFrage(f);
		}

		IO.println("\nPrueflinge eingeben");
		IO.println("-------------------");
		anzahl = IO.readInt("Anzahl Prueflinge: ");
		this.studenten = new Pruefling[anzahl];
		for (int i = 0; i < anzahl; i++) {
			this.studenten[i] = new Pruefling(IO.readString("Pruefling "
					+ (i + 1) + ": "));
		}
	}

	void durchfuehren() {
		IO.println("\nPruefung");
		IO.println("--------");
		// alle Prueflinge abfragen
		for (int i = 0; i < this.studenten.length; i++) {
			IO.println("\"" + this.studenten[i].getName()
					+ "\" ist an der Reihe");
			Frage f = null;
			// alle Fragen der Pruefung stellen
			while ((f = this.pruefung.liefereNaechsteFrage()) != null) {
				IO.println(f.getFrage());
				boolean antwort = IO.readChar("wahr/falsch (w/f)?") == 'w';
				if (f.isRichtig() == antwort) {
					IO.println("Hervorragend: Richtig beantwortet");
					this.studenten[i].neuePunkte(f.getPunkte());
				} else {
					IO.println("Niete: Falsch beantwortet");
				}
			}
		}
	}

	void ergebnisseBekanntgeben() {
		this.ranglisteErstellen();
		IO.println("\nPruefungsergebnisse");
		IO.println("-------------------");
		IO.println("Quiz: " + this.pruefung.getTitel());
		for (int i = 0; i < this.studenten.length; i++) {
			IO.println("Platz " + (i + 1) + ": " + this.studenten[i].getName()
					+ " mit " + this.studenten[i].getPunkte() + " Punkten");
		}
	}

	void ranglisteErstellen() {
		// Bubblesort nach erreichten Punkten
		boolean veraendert = false;
		do {
			veraendert = false;
			for (int i = 0; i < this.studenten.length - 1; i++) {
				if (this.studenten[i].getPunkte() < this.studenten[i + 1]
						.getPunkte()) {
					Pruefling help = this.studenten[i];
					this.studenten[i] = this.studenten[i + 1];
					this.studenten[i + 1] = help;
					veraendert = true;
				}
			}
		} while (veraendert);
	}

}