class Spielzug {
	int zeile; // 0-2

	int spalte; // 0-2
}

class TicTacToe {

	public static void main(String[] args) {
		int[][] brett = spielbrettInitialisieren();
		spielbrettAusgeben(brett);

		// abwechselndes Ziehen bis Spielende erreicht ist
		boolean aAmZug = true;
		while (true) {
			int ergebnis = spielzugDurchfuehren(brett, aAmZug);
			if (ergebnis >= 0) { // Ende
				ergebnisAusgeben(ergebnis);
				break;
			}
			aAmZug = !aAmZug; // Spielerwechsel
		}
	}

	static int[][] spielbrettInitialisieren() {
		int[][] brett = new int[3][3];
		for (int zeile = 0; zeile < 3; zeile++) {
			for (int spalte = 0; spalte < 3; spalte++) {
				brett[zeile][spalte] = 0;
				// 0 = leeres Feld; 1 = Spieler A; 2 = Spieler B
			}
		}
		return brett;
	}

	static void spielbrettAusgeben(int[][] brett) {
		IO.print("  ");
		for (int j = 0; j < 3; j++) {
			IO.print(j);
			IO.print(" ");
		}
		IO.println();

		IO.print("-");
		for (int j = 0; j < 3; j++)
			IO.print("--");
		IO.println("+");

		for (int i = 0; i < 3; i++) {
			IO.print(i);
			IO.print("|");
			for (int j = 0; j < 3; j++) {
				if (brett[i][j] == 0)
					IO.print(" ");
				else if (brett[i][j] == 1)
					IO.print("X");
				else
					IO.print("O");
				IO.print("|");
			}
			IO.println();

			IO.print("-");
			for (int j = 0; j < 3; j++)
				IO.print("--");
			IO.println("+");
		}
	}

	static void ergebnisAusgeben(int ergebnis) {
		if (ergebnis == 1) {
			IO.println("Spieler A hat gewonnen!");
		} else if (ergebnis == 2) {
			IO.println("Spieler B hat gewonnen!");
		} else if (ergebnis == 0) {
			IO.println("Unentschieden!");
		}
	}

	static int spielzugDurchfuehren(int[][] brett, boolean aAmZug) {
		spielerAusgeben(aAmZug);
		Spielzug zug = naechstenSpielzugErfragen(brett);
		spielzugAusfuehren(brett, zug, aAmZug);
		spielbrettAusgeben(brett);
		return spielEnde(brett);
	}

	static void spielerAusgeben(boolean aAmZug) {
		if (aAmZug)
			IO.println("Spieler A ist am Zug!");
		else
			IO.println("Spieler B ist am Zug!");
	}

	static Spielzug naechstenSpielzugErfragen(int[][] brett) {
		Spielzug zug = new Spielzug();
		zug.zeile = IO.readInt("Zeile (0-2): ");
		zug.spalte = IO.readInt("Spalte (0-2): ");
		while (!spielzugOK(brett, zug)) {
			IO.println("Ungueltig; neue Eingabe!");
			zug.zeile = IO.readInt("Zeile (0-2): ");
			zug.spalte = IO.readInt("Spalte (0-2): ");
		}
		return zug;
	}

	static boolean spielzugOK(int[][] brett, Spielzug zug) {
		// Koordinaten ok ?
		if (zug.zeile < 0 || zug.zeile > 2 || zug.spalte < 0 || zug.spalte > 2)
			return false;

		// schon besetzt ?
		if (brett[zug.zeile][zug.spalte] != 0)
			return false;

		return true;
	}

	static void spielzugAusfuehren(int[][] brett, Spielzug zug, boolean spielerA) {
		if (spielerA)
			brett[zug.zeile][zug.spalte] = 1;
		else
			brett[zug.zeile][zug.spalte] = 2;
	}

	static int spielEnde(int[][] brett) {
		if (horizontale(brett, 1) == 1 || vertikale(brett, 1) == 1
				|| diagonale(brett, 1) == 1) {
			return 1; // Sieger ist Spieler A
		}
		if (horizontale(brett, 2) == 2 || vertikale(brett, 2) == 2
				|| diagonale(brett, 2) == 2) {
			return 2; // Sieger ist Spieler B
		}
		if (allesBelegt(brett))
			return 0; // unentschieden
		return -1; // noch kein Spielende
	}

	static boolean allesBelegt(int[][] brett) {
		for (int zeile = 0; zeile < brett.length; zeile++)
			for (int spalte = 0; spalte < brett.length; spalte++)
				if (brett[zeile][spalte] == 0)
					return false;
		return true;
	}

	static int vertikale(int[][] brett, int spieler) {
		for (int r = 0; r < 3; r++) {
			int anzahl = 0;
			for (int s = 0; s < 3; s++) {
				if (brett[r][s] == spieler)
					anzahl++;
			}
			if (anzahl == 3)
				return spieler; // Sieger !
		}
		return -1;
	}

	static int horizontale(int[][] brett, int spieler) {
		for (int s = 0; s < 3; s++) {
			int anzahl = 0;
			for (int r = 0; r < 3; r++) {
				if (brett[r][s] == spieler)
					anzahl++;
			}
			if (anzahl == 3)
				return spieler; // Sieger !
		}
		return -1;
	}

	static int diagonale(int[][] brett, int spieler) {
		// links nach rechts
		int anzahl = 0;
		for (int f = 0; f < 3; f++) {
			if (brett[f][f] == spieler)
				anzahl++;
		}
		if (anzahl == 3)
			return spieler; // Sieger !

		// rechts nach links
		anzahl = 0;
		for (int f = 0; f < 3; f++) {
			if (brett[f][2 - f] == spieler)
				anzahl++;
		}
		if (anzahl == 3)
			return spieler; // Sieger !

		return -1;
	}
}
